#include "testlib.h"
#include <algorithm>
#include <vector>

typedef std::vector<std::vector<int> > matrix_t;
typedef std::vector<std::pair<int, size_t> > sorted_ind_t;

sorted_ind_t argsort(std::vector<int> const &list) {
    std::vector<std::pair<int, size_t> > ret(list.size());
    for (size_t i = 0; i != list.size(); ++i) {
        ret[i] = std::make_pair(list[i], i);
    }
    std::sort(ret.begin(), ret.end());
    return ret;
}

bool check(sorted_ind_t const &correct, sorted_ind_t const &to_check) {
    for (size_t i = 0; i != correct.size(); ++i) {
        if (correct[i].second != to_check[i].second) {
            return false;
        }
        
        if (i != correct.size() - 1 &&
            ((correct[i].first == correct[i + 1].first) !=
             (to_check[i].first == to_check[i + 1].first)))
        {
            return false;
        }
    }
    
    return true;
}

int read_positive(InStream &stream) {
    int val = stream.readInt();
    if (val < 1) {
        stream.quitf(_wa, "Elements must be positive, but found %d", val);
    }
    return val;
}

matrix_t read_matrix(InStream &stream, size_t n, int m, int &max_val) {
    max_val = 0;
    
    std::vector<std::vector<int> > matrix(n, std::vector<int>(m));
    for (size_t i = 0; i != n; ++i) {
        for (size_t j = 0; j != m; ++j) {
            matrix[i][j] = read_positive(stream);
            max_val = std::max(max_val, matrix[i][j]);
        }
    }
    return matrix;
}

std::vector<int> get_row(matrix_t const &matrix, size_t ind) {
    std::vector<int> row(matrix[ind].size());
    for (size_t i = 0; i != matrix[ind].size(); ++i) {
        row[i] = matrix[ind][i];
    }
    return row;
}

std::vector<int> get_column(matrix_t const &matrix, size_t ind) {
    std::vector<int> column(matrix.size());
    for (size_t i = 0; i != column.size(); ++i) {
        column[i] = matrix[i][ind];
    }
    return column;
}

int main(int argc, char **argv) {
    registerTestlibCmd(argc, argv);
    
    int n = inf.readInt();
    int m = inf.readInt();
    
    int ja, pa;
    
    std::vector<std::vector<int> > in_matrix = read_matrix(inf, n, m, ja);
    std::vector<std::vector<int> > judge_matrix = read_matrix(ans, n, m, ja);
    std::vector<std::vector<int> > part_matrix = read_matrix(ouf, n, m, pa);
    
    for (size_t i = 0; i != n; ++i) {
        if (!check(argsort(get_row(in_matrix, i)), argsort(get_row(part_matrix, i)))) {
            quitf(_wa, "Row %d order is changed", (int)i + 1);
        }
        if (!check(argsort(get_row(in_matrix, i)), argsort(get_row(judge_matrix, i)))) {
            quitf(_fail, "JE: Row %d order is changed", (int)i + 1);
        }
    }
    for (size_t i = 0; i != m; ++i) {
        if (!check(argsort(get_column(in_matrix, i)), argsort(get_column(part_matrix, i)))) {
            quitf(_wa, "Column %d order is changed", (int)i + 1);
        }
        if (!check(argsort(get_column(in_matrix, i)), argsort(get_column(judge_matrix, i)))) {
            quitf(_fail, "JE: Column %d order is changed", (int)i + 1);
        }
    }
    
    if (pa < ja) {
        quitf(_fail, "Participant answer %d, though judge is only %d", pa, ja);
    }
    if (pa > ja) {
        quitf(_wa, "Found max value %d, although better solution (%d) exists", pa, ja);
    }
    
    quitf(_ok, "Max value %d", pa);
}