#include "testlib.h"

#include <iostream>
#include <set>
#include <vector>
//#include <unordered_set>
#include <map>

int const MAX_VALUE = 1000 * 1000 * 1000;

struct DSU {
    std::vector<int> up, rank;
    std::vector<int> value;
    
    int get(int n) {
        return up[n] == n ? n : up[n] = get(up[n]);
    }
    
    DSU(int size) : up(size), rank(size, 0), value(size) {
        for (int i = 0; i != size; ++i) {
            up[i] = i;
        }
    }
    
    bool unite(int a, int b) {
        a = get(a);
        b = get(b);
        
        if (a == b) {
            return false;
        }
        
        if (rank[a] < rank[b]) {
            std::swap(a, b);
        }
        
        up[b] = a;
        if (rank[a] == rank[b]) {
            ++rank[a];
        }
        
        return true;
    }
    
    int get_value(int n) {
        return value[get(n)];
    }
    
    bool connected(int a, int b) {
        return get(a) == get(b);
    }
};

std::map<int, size_t> cnt = {{10000, 6421}, {8001, 6421}, {1000000, 823117}, {800001, 411527}};

std::vector<int> get_values(int size) {
// OS-specific values
/*
    std::unordered_set<int> set;
    for (int i = 0; i != size / 2; ++i) {
        set.insert(i);
    }
    size_t buckets_cnt = set.bucket_count();
*/
    
    ensure(cnt.find(size) != cnt.end());
    size_t buckets_cnt = cnt[size];
    
    std::set<int> res;
    
    for (size_t i = 0; res.size() != size; ++i) {
        res.insert((int)(buckets_cnt * i % MAX_VALUE + 1));
    }
    
    return std::vector<int>(res.begin(), res.end());
}

int main(int argc, char **argv) {
    registerGen(argc, argv, 1);
    
    ensuref(argc >= 6, "%d arguments, required 5", argc - 1);
    
    int minn = atoi(argv[1]);
    int maxn = atoi(argv[2]);
    int minm = atoi(argv[3]);
    int maxm = atoi(argv[4]);
    int equal_percent = atoi(argv[5]);
    
    int n = rnd.next(minn, maxn);
    int m = rnd.next(minm, maxm);
    int size = n * m;
    int iterations = (n * m - 1) * equal_percent / 100;
    DSU dsu(size);
    
    for (int i = 0; i != iterations; ++i) {
        int a, b;
        do {
            a = rnd.next(size);
            b = rnd.next(size);
        } while (dsu.connected(a, b));
        ensure(dsu.unite(a, b));
    }
    
    std::vector<int> vals = get_values(n * m - iterations);
    shuffle(vals.begin(), vals.end());
    std::vector<int>::const_iterator last = vals.begin();
    
    for (int i = 0; i != size; ++i) {
        if (dsu.up[i] == i) {
            dsu.value[i] = *last++;
        }
    }
    
    std::cout << n << ' ' << m << '\n';
    for (int i = 0; i != n; ++i) {
        for (int j = 0; j != m; ++j) {
            std::cout << dsu.get_value(i * m + j);
            if (j != m - 1) {
                std::cout << ' ';
            }
        }
        std::cout << '\n';
    }
    
    return 0;
}