#include <algorithm>
#include <cstdio>
#include <vector>
#include <map>
#include <set>

using namespace std;

typedef long long ll;

struct Vertex {
    int ans, size_in;
    char visited;
    vector<ll> edges;
    
    Vertex() : visited(0), size_in(0) {}
};

vector<ll> values;
vector<Vertex> verteces;

size_t ltoi(ll val) {
    return lower_bound(values.begin(), values.end(), val) - values.begin();
}

void make_edge(ll from, ll to) {
    size_t f = ltoi(from), t = ltoi(to);
    verteces[f].edges.push_back(t);
    ++verteces[t].size_in;
}

int main() {
    int n, m;
    scanf("%d %d", &n, &m);
    
    vector<vector<ll> > matrix(n, vector<ll>(m));
    values.resize(n * m);
    
    for (int i = 0; i != n; ++i) {
        for (int j = 0; j != m; ++j) {
            scanf("%lld", &matrix[i][j]);
            values[i * m + j] = matrix[i][j];
        }
    }
    
    sort(values.begin(), values.end());
    values.erase(unique(values.begin(), values.end()), values.end());
    verteces.resize(values.size());
    
    for (int i = 0; i != n; ++i) {
        for (int j = 0; j != m; ++j) {
            for (int ii = 0; ii != n; ++ii) {
                if (ii != i && matrix[i][j] < matrix[ii][j]) {
                    make_edge(matrix[i][j], matrix[ii][j]);
                }
            }
            for (int jj = 0; jj != m; ++jj) {
                if (jj != j && matrix[i][j] < matrix[i][jj]) {
                    make_edge(matrix[i][j], matrix[i][jj]);
                }
            }
        }
    }
    
    vector<ll> curr, next;
    
    for (size_t i = 0; i != verteces.size(); ++i) {
        if (!verteces[i].size_in) {
            curr.push_back(i);
        }
    }
    
    int level = 1;
    while (!curr.empty()) {
        while (!curr.empty()) {
            Vertex &vertex = verteces[curr.back()];
            curr.pop_back();
            
            vertex.ans = level;
            
            for (vector<ll>::const_iterator it = vertex.edges.begin(); it != vertex.edges.end(); ++it) {
                Vertex &to = verteces[*it];
                if (!(--to.size_in)) {
                    next.push_back(*it);
                }
            }
        }
        
        ++level;
        curr.swap(next);
    }
    
    //printf("%d\n", level - 1);
    for (int i = 0; i != n; ++i) {
        for (int j = 0; j != m; ++j) {
            printf("%d ", verteces[ltoi(matrix[i][j])].ans);
        }
        printf("\n");
    }
    
    return 0;
}