#include "testlib.h"
#include <cassert>
#include <functional>
using namespace std;

map<string, function<char**(int, char**)>> generators;

#define ADD_GENERATOR(name) \
char** gen_##name(int n, char* argv[]); \
int notused_##name = (generators[#name] = gen_##name, -42); \
char** gen_##name(int n, char* argv[])

vector<pair<int, int>> edges;

ADD_GENERATOR(rand) {
    for (int i = 2; i <= n; i++) {
        edges.emplace_back(i, rnd.next(1, i - 1));
    }
    return argv;
}

ADD_GENERATOR(chain) {
    for (int i = 2; i <= n; i++) {
        edges.emplace_back(i - 1, i);
    }
    return argv;
}

ADD_GENERATOR(star) {
    for (int i = 2; i <= n; i++) {
        edges.emplace_back(1, i);
    }
    return argv;
}

char** gen(int n, char** argv) {
    auto it = generators.find(argv[0]);
    assert(it != generators.end());
    return it->second(n, argv + 1);
}

const int N = 100500;
vector<int> E[N];
bool used[N];

void DFS(int x, int p = -1) {
    used[x] = true;
    for (int y : E[x]) {
        if (y == p)
            continue;
        DFS(y, x);
    }
}

int main(int argc, char* argv[]) {
    registerGen(argc, argv, 1);
    ++argv;
    int n = atoi(argv[0]);
    printf("%d\n", n);
    ++argv;
    argv = gen(n, argv);
    for (auto e : edges) {
        printf("%d %d\n", e.first, e.second);
    }
    pair<int, int> e = edges[rnd.next(0, n - 2)];
    edges.erase(find(edges.begin(), edges.end(), e));
    for (auto e : edges) {
        E[e.first].push_back(e.second);
        E[e.second].push_back(e.first);
    }
    DFS(e.first);
    while (true) {
        int a = rnd.next(1, n);
        int b = rnd.next(1, n);
        if (make_pair(a, b) == e || make_pair(b, a) == e || used[a] == used[b])
            continue;
        edges.insert(edges.begin() + rnd.next(0, n - 2), make_pair(a, b));
        break;
    }
    for (auto e : edges) {
        printf("%d %d\n", e.first, e.second);
    }
}


